# Generated by Django 2.2.28 on 2022-04-12 13:59

from django.db import migrations
from django.db import models

import redisboard.models


class Migration(migrations.Migration):

    dependencies = [
        ('redisboard', '0003_fill_url'),
    ]

    operations = [
        migrations.AlterField(
            model_name='redisserver',
            name='password',
            field=models.CharField(
                blank=True,
                help_text='You can also specify the password here (the field is masked).',
                max_length=250,
                null=True,
                verbose_name='Password',
            ),
        ),
        migrations.AlterField(
            model_name='redisserver',
            name='url',
            field=models.CharField(
                default='missing',
                help_text='<a href="https://www.iana.org/assignments/uri-schemes/prov/redis">IANA-compliant</a> URL. Examples: <pre>'
                'redis://[[username]:[password]]@localhost:6379/0\n'
                'rediss://[[username]:[password]]@localhost:6379/0\n'
                'unix://[[username]:[password]]@/path/to/socket.sock?db=0</pre>',
                max_length=250,
                unique=True,
                validators=[redisboard.models.validate_url],
                verbose_name='URL',
            ),
            preserve_default=False,
        ),
        migrations.AlterUniqueTogether(
            name='redisserver',
            unique_together=set(),
        ),
        migrations.RemoveField(
            model_name='redisserver',
            name='hostname',
        ),
        migrations.RemoveField(
            model_name='redisserver',
            name='port',
        ),
    ]
